import java.awt.image.BufferedImage;

public class PixelBufferFilter2AsBufferedImageFilter implements BufferedImageFilter {
	private PixelBufferFilter2 pixelBufferFilter;
	private int[] inPixelBuffer;
	private int[] outPixelBuffer;
	
	public PixelBufferFilter2AsBufferedImageFilter(PixelBufferFilter2 pixelBufferFilter) {
		this.pixelBufferFilter = pixelBufferFilter;
		this.inPixelBuffer = null;
		this.outPixelBuffer = null;
	}
	
	public void filter(BufferedImage img) {
		int width = img.getWidth();
		int height = img.getHeight();
		
		// Copy the image's pixels into the input pixel buffer
		this.inPixelBuffer = BufferedImages.copyBufferedImageToARGBPixelBuffer(img, this.inPixelBuffer);
		
		// Ensure output pixel buffer is size-compatible with the input image
		this.outPixelBuffer = BufferedImages.createSizeCompatiblePixelBuffer(img, this.outPixelBuffer);
		
		// Filter the contents of the pixel buffer
		pixelBufferFilter.filter(width, height, inPixelBuffer, outPixelBuffer);
		
		// Store the pixel buffer back into the original image
		BufferedImages.setAllRGB(img, outPixelBuffer);
	}
	
	public String toString() {
		return pixelBufferFilter.toString();
	}
}