// EXPERIMENTAL
// NOTE: this filter ignores the input image
public class LineGenerator implements PixelBufferFilter, ParameterizedFilter {
	private final BooleanFilterParam axisParam =
		new BooleanFilterParam("Axis (on=X, off=Y)", false);
		
	private final IntFilterParam slopeDenomParam =
		new IntFilterParam("Slope Denominator (1/x)", Integer.MIN_VALUE, Integer.MAX_VALUE, 1);
	
	private final IntFilterParam interceptParam =
		new IntFilterParam("X/Y Intercept", Integer.MIN_VALUE, Integer.MAX_VALUE, 50);
	
	private final FilterParam[] params = new FilterParam[] {
		axisParam,
		slopeDenomParam,
		interceptParam
	};
	
	// ### FILTER ###
	
	private final static int BKGND_PIXEL = Pixels.BLACK_ARGB_PIXEL;
	private final static int LINE_PIXEL = Pixels.WHITE_ARGB_PIXEL;
	
	public void filter(int width, int height, int[] pixels) {
		boolean axis = axisParam.getValue();	// true=X, false=Y
		int slopeDenom = slopeDenomParam.getValue();
		int intercept = interceptParam.getValue();
		
		int numPixels = pixels.length;
		
		/* Clear output image to white */
		for (int i=0; i<numPixels; i++) {
			pixels[i] = BKGND_PIXEL;
		}
		
		/* Draw thin black line with the specified slope, intercept, and axis*/
		if (axis == false) {
			/* Y Axis */
			if (slopeDenom > 0) {
				int x = 0;
				for (int y=intercept, off=y*width+x;
					 y<height && x<width;
					 y++, off+=width)
				{
					if (y < 0) {
						/* y is out of bounds (too small) */
						x += slopeDenom;
						off += slopeDenom;
					} else {
						/* y is in bounds */
						for (int i=slopeDenom;
							 i>0 && x<width;
							 i--, x++, off++)
						{
							pixels[off] = LINE_PIXEL;
						}
					}
				}
			} else if (slopeDenom < 0) {
				slopeDenom = -slopeDenom;
				
				int x = 0;
				for (int y=intercept, off=y*width+x;
					 y>=0 && x<width;
					 y--, off-=width)
				{
					if (y >= height) {
						/* y is out of bounds (too big) */
						x += slopeDenom;
						off += slopeDenom;
					} else {
						/* y is in bounds */
						for (int i=slopeDenom;
							 i>0 && x<width;
							 i--, x++, off++)
						{
							pixels[off] = LINE_PIXEL;
						}
					}
				}
			} else if (slopeDenom == 0) {
				// can't draw a slope-intercept line with a slope of infinity
			}
		} else {
			/* X Axis */
			if (slopeDenom > 0) {
				int y = 0;
				int slopeDenom_times_width = slopeDenom*width;
				for (int x=intercept, off=y*width+x;
					 x<width && y<height;
					 x++, off++)
				{
					if (x < 0) {
						/* x is out of bounds (too small) */
						y += slopeDenom;
						off += slopeDenom_times_width;
					} else {
						/* x is in bounds */
						for (int i=slopeDenom;
							 i>0 && y<height;
							 i--, y++, off+=width)
						{
							pixels[off] = LINE_PIXEL;
						}
					}
				}
			} else if (slopeDenom < 0) {
				slopeDenom = -slopeDenom;
				
				int y = 0;
				int slopeDenom_times_width = slopeDenom*width;
				for (int x=intercept, off=y*width+x;
					 x>=0 && y<height;
					 x--, off--)
				{
					if (x >= width) {
						/* x is out of bounds (too big) */
						y += slopeDenom;
						off += slopeDenom_times_width;
					} else {
						/* x is in bounds */
						for (int i=slopeDenom;
							 i>0 && y<height;
							 i--, y++, off+=width)
						{
							pixels[off] = LINE_PIXEL;
						}
					}
				}
			} else if (slopeDenom == 0) {
				// can't draw a slope-intercept line with a slope of infinity
			}
		}
	}
	
	public String toString() {
		return "Line Generator #1";
	}
	
	// ### PARAMETERIZED FILTER ###
	
	public FilterParam[] getParams() {
		return params;
	}
}